/*	$Csoft$	*/
/*	Public domain	*/

#define RCSINFO_MSG_MAX	16384
#define RCSINFO_NAME_MAX 1024			/* Sync with agar object.h */
#define RCSINFO_AUTHOR_MAX USER_NAME_MAX
#define RCSINFO_DIGEST_MAX 160
#define PROG_CURRENT_REVISION 0

int prog_commit(NS_Command *, void *);
int prog_fetch(NS_Command *, void *);
int prog_list(NS_Command *, void *);

