/*	$Csoft: billingd.h,v 1.1.1.1 2005/03/25 02:29:22 vedge Exp $	*/
/*	Public domain	*/

#include <sys/types.h>

#include <agar/core.h>
#include <agar/net.h>

typedef unsigned int Uint;

#define Malloc AG_Malloc
#define Free AG_Free

#include "user.h"

extern struct user *user;
